"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const definition_1 = require('./definition');
class BaseMapper {
    constructor(knexBuilder, options) {
        this.knexBuilder = knexBuilder;
        this.options = options;
    }
    selectAllFrom(model) {
        let tableName = definition_1.getTableName(model);
        let fieldNames = definition_1.getFieldNames(model);
        let absoluteFieldNames = fieldNames.map(fieldName => `${tableName}.${fieldName}`);
        let query = this.knexBuilder.select(absoluteFieldNames).from(tableName);
        return new SelectQuery(model, query, this.options);
    }
    updateWith(model, data) {
        let tableName = definition_1.getTableName(model);
        let fieldData = definition_1.serializeData(model, data, this.options);
        let query = this.knexBuilder.table(tableName).update(fieldData);
        return new UpdateQuery(model, query, this.options);
    }
    insertInto(model, data) {
        let query = this.knexBuilder
            .insert(definition_1.serializeData(model, data, this.options))
            .into(model.__metadata.tableName);
        return new InsertQuery(model, query, this.options);
    }
    truncate(model) {
        return Promise.resolve(this.knexBuilder.table(model.__metadata.tableName).truncate());
    }
    tryFindOneByKey(model, key) {
        return __awaiter(this, void 0, void 0, function* () {
            let attributeNames = definition_1.getAttributes(model);
            let whereConditions = definition_1.serializeData(model, key, this.options);
            let fieldNames = definition_1.getFieldNames(model);
            let query = this.knexBuilder.columns(fieldNames).select().from(model.__metadata.tableName);
            if (Object.keys(whereConditions).length > 0) {
                query = query.where(whereConditions);
            }
            let data = yield query.limit(1);
            if (data.length === 1) {
                return definition_1.deserializeData(model, data[0], this.options);
            }
            else {
                return null;
            }
        });
    }
    findOneByKey(model, key) {
        return __awaiter(this, void 0, void 0, function* () {
            let data = yield this.tryFindOneByKey(model, key);
            if (!data) {
                throw new Error("Expected to find exactly one row");
            }
            else {
                return data;
            }
        });
    }
}
exports.BaseMapper = BaseMapper;
class Mapper extends BaseMapper {
    //private query: knex.QueryBuilder;
    constructor(knexClient, options) {
        super(knexClient, options);
        this.knexClient = knexClient;
    }
    transaction(callback) {
        return Promise.resolve(this.knexClient.transaction((trx) => {
            let t;
            let transactionMapper = new BaseMapper(trx, this.options);
            return callback(transactionMapper);
        }));
    }
}
exports.Mapper = Mapper;
class BaseQuery {
    constructor(knexQuery) {
        this.knexQuery = knexQuery;
    }
    getKnexQuery() {
        return this.knexQuery;
    }
}
exports.BaseQuery = BaseQuery;
class WhereQuery extends BaseQuery {
    constructor(knexQuery) {
        super(knexQuery);
    }
    where(clause) {
        throw new Error("Not implemented");
    }
    whereEqual(attribute, value) {
        return this.whereOperator(attribute, '=', value);
    }
    whereLess(attribute, value) {
        return this.whereOperator(attribute, '<', value);
    }
    whereLessOrEqual(attribute, value) {
        return this.whereOperator(attribute, '<=', value);
    }
    whereGreater(attribute, value) {
        return this.whereOperator(attribute, '>', value);
    }
    whereGreaterOrEqual(attribute, value) {
        return this.whereOperator(attribute, '>=', value);
    }
    whereOperator(attribute, operator, value) {
        let attributeDefinition = attribute;
        this.knexQuery = this.knexQuery.andWhere(attributeDefinition.fieldName, operator, value);
        return this;
    }
}
exports.WhereQuery = WhereQuery;
class SelectQuery extends WhereQuery {
    constructor(resultModel, knexQuery, serializationOptions) {
        super(knexQuery);
        this.resultModel = resultModel;
        this.serializationOptions = serializationOptions;
    }
    forUpdate() {
        this.knexQuery = this.knexQuery.forUpdate();
        return this;
    }
    limit(count) {
        this.knexQuery = this.knexQuery.limit(count);
        return this;
    }
    orderBy(attribute, direction) {
        let attributeDefinition = attribute;
        this.knexQuery = this.knexQuery.orderBy(attributeDefinition.fieldName, direction);
        return this;
    }
    getOne() {
        return __awaiter(this, void 0, void 0, function* () {
            let data = yield this.execute();
            if (data.length === 1) {
                return data[0];
            }
            else {
                throw new Error("Expected exactly one row");
            }
        });
    }
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            let queryResults = yield this.knexQuery;
            return queryResults.map(result => definition_1.deserializeData(this.resultModel, result, this.serializationOptions));
        });
    }
    then(onfulfilled, onrejected) {
        return this.execute().then(onfulfilled, onrejected);
    }
}
exports.SelectQuery = SelectQuery;
class UpdateQuery extends WhereQuery {
    constructor(model, knexQuery, serializationOptions) {
        super(knexQuery);
        this.model = model;
        this.serializationOptions = serializationOptions;
    }
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            let queryResults = yield this.knexQuery;
            return;
        });
    }
    then(onfulfilled, onrejected) {
        return this.knexQuery.then(onfulfilled, onrejected);
    }
}
exports.UpdateQuery = UpdateQuery;
class InsertQuery extends BaseQuery {
    constructor(model, knexQuery, serializationOptions) {
        super(knexQuery);
        this.model = model;
        this.serializationOptions = serializationOptions;
    }
    execute() {
        return __awaiter(this, void 0, void 0, function* () {
            let queryResults = yield this.knexQuery;
            return;
        });
    }
    then(onfulfilled, onrejected) {
        return this.knexQuery.then(onfulfilled, onrejected);
    }
}
exports.InsertQuery = InsertQuery;
//# sourceMappingURL=core.js.map